﻿namespace U2UConsult.Win8.DocumentLibrarySample.Models
{
    using System;
    using System.Collections.ObjectModel;
    using System.IO;
    using System.Linq;
    using System.Threading.Tasks;
    using System.Xml.Serialization;
    using Windows.Storage;
    using Windows.Storage.AccessCache;

    public static class AppModel
    {
        // Name of the subfolder in the user's Documents folder that is reserved for this app.
        private static string AppFolderName = "Trappist Beers";

        // Extension for the files of this app.
        private static string AppFileExtension = ".trappist";

        /// <summary>
        /// Returns the subfolder in the user's Documents folder that is reserved for this app.
        /// </summary>
        public static async Task<StorageFolder> AppFolder()
        {
            return await KnownFolders.DocumentsLibrary.GetFolderAsync(AppFolderName);
        }

        /// <summary>
        /// Creates and populates a folder for this app in the user's Documents Folder.
        /// </summary>
        public static async Task EnsureFolderExists()
        {
            StorageFolder folder = null;

            try
            {
                folder = await KnownFolders.DocumentsLibrary.GetFolderAsync(AppFolderName);
            }
            catch (Exception)
            {
                // Folder does not exist, but we cannot await in a catch block, so we can not create it here.
            }

            if (folder == null)
            {
                folder = await KnownFolders.DocumentsLibrary.CreateFolderAsync(AppFolderName, CreationCollisionOption.OpenIfExists);

                // Create some initial content
                Trappist orval = new Trappist()
                    {
                        Name = "Orval",
                        Brewery = "Orval Brewery is a Belgian trappist brewery located within the walls of the Abbaye Notre-Dame d'Orval in the Gaume region of Belgium. The brewery is normally closed to the public, but opens its doors for two days each year. There are presently 32 secular workers.",
                        History = "Evidence of brewing goes back to the earliest days of the monastery. A document written by the abbot in 1628 directly refers to the consumption of beer and wine by the monks. The last of the brewers to be a monk was Brother Pierre, up until the 1793 fire. In 1931 the present day brewery was built, employing lay people and intended to provide a source of funds for the monastery reconstruction. It was designed by Henry Vaes, who also designed the distinctive Orval beer glass. The first beer was shipped from the brewery on 7 May 1932, and was sold in barrels rather than the bottles of today. Orval was the first Trappist beer to be sold nationally around Belgium.",
                        Description = "Orval is the main brand. It is a 6.9% abv beer. It has a complex and unusual flavor and aroma produced by a unique strain of yeast. The beer is light in color, slightly cloudy, and has a large, foamy head. There is a complex aroma of leather, spice, and many other earthy components. Its very distinctive taste is largely attributed to two parts of the brewing process. One of these is the use of dry hopping, in which large meshed bags of hops infuse the beer during the three week maturation period. The other is the use of Brettanomyces yeast during this same maturation, which are a local wild yeast. Hallertau, Styrian Goldings and French Strisselspalt hops are used. Orval beer is bottled exclusively in a distinctive skittle shaped 33 cl bottle. The beer is then matured at 15°C for a minimum of four weeks on site before being distributed. As the beer is bottle conditioned, its flavour can improve over the years with ageing."
                    };
                await orval.Save();

                Trappist dubbel = new Trappist()
                    {
                        Name = "Westmalle Dubbel",
                        Brewery = "The Trappist brewery of Westmalle is located within the walls of the abbey in its entirety, and for this reason, it is not accessible to visitors.",
                        History = "On 22 April 1836 the monastery became a Trappist abbey. Since then the monks have been able to drink the popular beverage of the region with their meals. And in Flanders that is... beer. Thus in the same year abbot Martinus Dom started construction of a small brewery. On 10 December 1836 they served their first brew of Trappist beer at lunch. For many years the abbey only brewed for its own needs. Only as of 1856 did the monks sell some beer at the gate now and again. Demand increased year on year, until the brewery had to expand in 1865 and 1897.",
                        Description = "Westmalle Dubbel is a dark, reddish-brown Trappist beer with a secondary fermentation in the bottle. The creamy head has the fragrance of special malt and leaves an attractive lace pattern in the glass. The flavour is rich and complex, herby and fruity with a fresh-bitter finish. It is a balanced quality beer with a soft feel in the mouth and a long, dry aftertaste. The Dubbel contains 7% alcohol."
                    };
                await dubbel.Save();

                Trappist rochefort = new Trappist()
                    {
                        Name = "Rochefort",
                        Brewery = "The brewery is located inside the Abbey of Notre-Dame de Saint-Rémy, near the town of Rochefort, and has been brewing beer since 1595. There are approximately 15 monks resident at the monastery.",
                        History = "Around 1230, Gilles de Walcourt, count of Rochefort founded a monastery for Cistercian nuns called Secours de Notre-Dame. In 1464 Louis de la Marck ordered the nuns to leave the monastery which had decayed and they were replaced by monks. The monastery was the latest Daughter-house of the abbey of Abbey of Cîteaux. During the Eighty Years War the abbey was ravaged by the Protestant armies of the Seventeen Provinces (1568) and the Austrian armies of John of Austria (1577). Around 1595, the first brewery was founded within the abbey. In the 17th century the abbey suffered from war, famine and the plague. On 30 April 1650, an army from Lorraine, led by baron Châtelet, invaded the abbey. The monks had to flee to Marche, as well as in 1652 and 1653. In 1789 the French revolutionary army invaded the Austrian Netherlands, and in 1797 the abbey was closed and sold to Lucien-Joseph Poncelet. Poncelet demolished the abbey around 1805 and converted it to a farm. Material of the abbey was used for buildings in Rochefort. On 11 October 1887, father Anselmus Judong from the Trappist Abbey of Achel came to the old abbey and on 21 December 1887 the buildings were bought by the monks of Achel. The abbey was restored and new buildings were raised. A new brewery was founded, but it would take until 1952 for the brewery to produce enough beer to be sold.",
                        Description = "Rochefort 10 has a dark brown color. Great strength balanced by a complexity of flavors and firm malt backbone. The bouquet covers a wide range: port wine, leather, apricots, oak, spices - a deeply intriguing beverage."
                    };
                await rochefort.Save();
            }
        }

        /// <summary>
        /// Extension Method to save a trappist.
        /// </summary>
        public async static Task Save(this Trappist trappist)
        {
            var file = await GetFile(trappist.Name);

            if (file == null)
            {
                // File doesn't exist yet. So let's create it.
                var folder = await AppFolder();
                file = await folder.CreateFileAsync(trappist.Name + AppFileExtension, CreationCollisionOption.ReplaceExisting);
            }

            // Content
            XmlSerializer serializer = new XmlSerializer(typeof(Trappist));
            using (StringWriter writer = new StringWriter())
            {
                serializer.Serialize(writer, trappist);
                String xmlString = writer.ToString();
                await FileIO.WriteTextAsync(file, xmlString);
            }

            // Move to top of MRU list
            StorageApplicationPermissions.MostRecentlyUsedList.Add(file, trappist.Name);
        }

        /// <summary>
        /// Deletes the file containing a Trappist beer, and removes it from the MRU list.
        /// </summary>
        public async static Task Delete(this Trappist trappist)
        {
            var file = await GetFile(trappist.Name);

            try
            {
                if (file != null)
                {
                    await file.DeleteAsync();
                }
            }
            catch (Exception)
            {
                // Ignore
            }
            finally
            {
                // MRU list
                StorageApplicationPermissions.MostRecentlyUsedList.Remove(GetMruToken(trappist.Name));
            }
        }

        /// <summary>
        /// Returns a deserialized Trappist from a file.
        /// </summary>
        public async static Task<Trappist> Open(string name)
        {
            Trappist trappist = null;
            var file = await GetFile(name);

            if (file != null)
            {
                // Content
                string xml = await FileIO.ReadTextAsync(file);
                XmlSerializer serializer = new XmlSerializer(typeof(Trappist));
                trappist = serializer.Deserialize(new StringReader(xml)) as Trappist;

                // MRU list
                StorageApplicationPermissions.MostRecentlyUsedList.Add(file, name);
            }

            return trappist;
        }

        /// <summary>
        /// Returns the most recently used list.
        /// </summary>
        public static ObservableCollection<string> MostRecentTrappists
        {
            get
            {
                var result = new ObservableCollection<string>();

                foreach (var item in StorageApplicationPermissions.MostRecentlyUsedList.Entries)
                {
                    result.Add(item.Metadata);
                }

                return result;
            }
        }

        /// <summary>
        /// Returns the list of Trappists from the app folder.
        /// </summary>
        /// <returns></returns>
        public async static Task<ObservableCollection<string>> FolderTrappists()
        {
            var result = new ObservableCollection<string>();

            var folder = await AppFolder();
            var files = await folder.GetFilesAsync();
            var names = from file in files
                        where file.FileType == AppFileExtension
                        orderby file.DisplayName
                        select file.DisplayName;

            foreach (var name in names)
            {
                result.Add(name);
            }

            return result;
        }

        /// <summary>
        /// Gets the name of the most recently opened Trappist.
        /// </summary>
        public static string MostRecentTrappistName
        {
            get
            {
                if (StorageApplicationPermissions.MostRecentlyUsedList.Entries.Count > 0)
                {
                    return StorageApplicationPermissions.MostRecentlyUsedList.Entries[0].Metadata;
                }

                return null;
            }
        }

        /// <summary>
        /// Gets the MRU token of a Trappist name.
        /// </summary>
        private static string GetMruToken(string trappistName)
        {
            var token = (from e in StorageApplicationPermissions.MostRecentlyUsedList.Entries
                         where e.Metadata == trappistName
                         select e.Token).FirstOrDefault();
            return token;
        }

        /// <summary>
        /// Gets the file where a Trappist is saved.
        /// </summary>
        private async static Task<StorageFile> GetFile(string trappistName)
        {
            StorageFile file = null;

            var token = GetMruToken(trappistName);

            if (!String.IsNullOrEmpty(token))
            {
                try
                {
                    // Get file through MRU list
                    file = await StorageApplicationPermissions.MostRecentlyUsedList.GetFileAsync(token);
                    return file;
                }
                catch (Exception)
                {
                    // File was probably deleted, so continue with app folder.
                }
            }

            // Get file from app folder
            try
            {
                var folder = await AppFolder();
                file = await folder.GetFileAsync(trappistName + AppFileExtension);
            }
            catch (Exception)
            {
                // Ignore
            }

            return file;
        }
    }
}