﻿namespace U2UConsult.Win8.DocumentLibrarySample.ViewModels
{
    using System;
    using System.Collections.ObjectModel;
    using System.Windows.Input;
    using Common;
    using U2UConsult.Win8.DocumentLibrarySample.Models;
    using Windows.UI.Popups;
    using WinRTXamlToolkit.Controls;

    public class MainPageViewModel : BindableBase
    {
        private string trappistName;
        private Trappist trappist;
        private ICommand newCommand;
        private ICommand renameCommand;
        private ICommand deleteCommand;
        private bool showAll = false;
        private ObservableCollection<string> trappistNames;

        private event EventHandler TrappistChanged;

        private event EventHandler TrappistNamesChanged;

        public MainPageViewModel()
        {
            // Initialization
            this.newCommand = new RelayCommand(this.New_Executed);
            this.renameCommand = new RelayCommand(this.Rename_Executed);
            this.deleteCommand = new RelayCommand(this.Delete_Executed);
            this.TrappistChanged += this.MainPageViewModel_TrappistChanged;
            this.TrappistNamesChanged += this.MainPageViewModel_TrappistNamesChanged;
            this.trappistNames = AppModel.MostRecentTrappists;

            // Open most recently used
            this.TrappistName = AppModel.MostRecentTrappistName;
        }

        /// <summary>
        /// Refreshes the list of Trappist names.
        /// </summary>
        private async void MainPageViewModel_TrappistNamesChanged(object sender, EventArgs e)
        {
            if (this.ShowAll)
            {
                this.trappistNames = await AppModel.FolderTrappists();
            }
            else
            {
                this.trappistNames = AppModel.MostRecentTrappists;
            }

            this.OnPropertyChanged("TrappistNames");
        }

        /// <summary>
        /// Gets or sets a value indicating whether Trappist names are read from the directory, or not.
        /// </summary>
        public bool ShowAll
        {
            get { return this.showAll; }
            set
            {
                this.SetProperty(ref this.showAll, value);
                this.TrappistNamesChanged(this, EventArgs.Empty);
                this.Trappist = null;
            }
        }

        /// <summary>
        /// Gets the 'New' Command.
        /// </summary>
        public ICommand NewCommand
        {
            get { return this.newCommand; }
            private set { this.newCommand = value; }
        }

        /// <summary>
        /// Gets the 'Rename' Command.
        /// </summary>
        public ICommand RenameCommand
        {
            get { return this.renameCommand; }
            private set { this.renameCommand = value; }
        }

        /// <summary>
        /// Gets the 'Delete' Command.
        /// </summary>
        public ICommand DeleteCommand
        {
            get { return this.deleteCommand; }
            private set { this.deleteCommand = value; }
        }

        /// <summary>
        /// Gets or sets the name of the current Trappist.
        /// </summary>
        public string TrappistName
        {
            get { return this.trappistName; }
            set
            {
                this.SetProperty(ref this.trappistName, value);
                if (this.trappistName != null)
                {
                    this.TrappistChanged(this, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Gets the current Trappist.
        /// </summary>
        public Trappist Trappist
        {
            get { return this.trappist; }
            private set
            {
                if (this.trappist != null)
                {
                    this.trappist.PropertyChanged -= this.Trappist_PropertyChanged;
                }

                this.SetProperty(ref this.trappist, value);
                if (this.trappist != null)
                {
                    this.trappist.PropertyChanged += this.Trappist_PropertyChanged;
                }
            }
        }

        /// <summary>
        /// Saves the Trappist when a property changed.
        /// </summary>
        private async void Trappist_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            if (e.PropertyName != "Name")
            {
                await this.Trappist.Save();
            }
        }

        /// <summary>
        /// Gets the list of Trappist names.
        /// </summary>
        public ObservableCollection<string> TrappistNames
        {
            get
            {
                return this.trappistNames;
            }
        }

        /// <summary>
        /// Opens the selected Trappist
        /// </summary>
        private async void MainPageViewModel_TrappistChanged(object sender, EventArgs e)
        {
            this.Trappist = await AppModel.Open(this.trappistName);
        }

        /// <summary>
        /// Creates a new Trappist.
        /// </summary>
        private async void New_Executed()
        {
            var dialog = new InputDialog();
            var result = await dialog.ShowAsync(
                "Add new Trappist",
                "Please enter the name:",
                "Create",
                "Cancel");

            if (result == "Create" && !String.IsNullOrEmpty(dialog.InputText))
            {
                // Check if exists
                var all = await AppModel.FolderTrappists();
                if (all.Contains(dialog.InputText) || AppModel.MostRecentTrappists.Contains(dialog.InputText))
                {
                    var messageDialog = new MessageDialog("This name already exists, please use another.");
                    await messageDialog.ShowAsync();
                    return;
                }

                Trappist newTrappist = new Trappist()
                {
                    Name = dialog.InputText
                };

                await newTrappist.Save();
                this.TrappistNamesChanged(this, EventArgs.Empty);
                this.TrappistName = newTrappist.Name;
            }
        }

        /// <summary>
        /// Renames a Trappist.
        /// </summary>
        private async void Rename_Executed()
        {
            var dialog = new InputDialog();
            var result = await dialog.ShowAsync(
                "Rename Trappist",
                "Please enter the new name:",
                "Rename",
                "Cancel");

            if (result == "Rename" && !String.IsNullOrEmpty(dialog.InputText))
            {
                // Check if exists
                var all = await AppModel.FolderTrappists();
                if (all.Contains(dialog.InputText) || AppModel.MostRecentTrappists.Contains(dialog.InputText))
                {
                    var messageDialog = new MessageDialog("This name already exists, please use another.");
                    await messageDialog.ShowAsync();
                    return;
                }

                await this.Trappist.Delete(); // Removes fom MRU
                this.Trappist.Name = dialog.InputText;
                await this.Trappist.Save();
                this.TrappistNamesChanged(this, EventArgs.Empty);

                this.TrappistName = dialog.InputText;
            }
        }

        /// <summary>
        /// Deletes a Trappist.
        /// </summary>
        private async void Delete_Executed()
        {
            await this.Trappist.Delete();

            this.TrappistNamesChanged(this, EventArgs.Empty);

            // Open most recently used
            this.TrappistName = AppModel.MostRecentTrappistName;
        }
    }
}